"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require('os');
const crypto = require('crypto-js');
const fse = require('fs-extra');
const path = require('path');
const homeDir = require('home-dir');
const packageJSON = require('../../package.json');
const { v4: uuidv4 } = require('uuid');
const api_client_1 = require("../api-client");
const uris_1 = require("../uris");
const INSTALLMENT_ID_FILE = path.join(homeDir(), '.anypoint/.installmentId');
async function generateInstallmentId() {
    const uniqueId = uuidv4();
    const osStamp = calculateOsStamp();
    const newInstallmentId = `${uniqueId}-${osStamp}`;
    if (!fse.existsSync(path.join(homeDir(), '.anypoint'))) {
        fse.mkdirSync(path.join(homeDir(), '.anypoint'));
    }
    await fse.writeFile(INSTALLMENT_ID_FILE, newInstallmentId, 'utf8');
    return newInstallmentId;
}
const loadInstallmentId = async () => {
    return fse.readFile(INSTALLMENT_ID_FILE, 'utf8')
        .then((data) => data)
        .catch(() => {
        return '';
    });
};
function getPlatformTraits() {
    var _a;
    const pversion = process.version || 'v4.0.0';
    return {
        cliVersion: packageJSON.version,
        os: os.type(),
        osVersion: os.release(),
        osPlatform: os.platform(),
        nodeVersion: pversion,
        nodeVersionMajor: (_a = ''.concat(pversion).split('.').shift()) === null || _a === void 0 ? void 0 : _a.replace('v', ''),
    };
}
function calculateOsStamp() {
    const osTraits = getPlatformTraits();
    const hash = crypto.SHA512(`${osTraits.os}${osTraits.osVersion}${osTraits.nodeVersion}`);
    return hash.toString(crypto.enc.Hex);
}
class Analytics {
    constructor(session) {
        this.session = session;
        this.installmentId = '';
        this.osTraits = getPlatformTraits();
    }
    getUserProperties() {
        return {
            rootOrganizationId: this.session.account.user.organization.id,
            organizationId: this.session.selectedOrganization.id,
            id: this.session.account.user.id,
        };
    }
    async getInstallmentId() {
        // if (!this.session.opts.collectMetrics) return
        if (this.installmentId)
            return this.installmentId;
        this.installmentId = await loadInstallmentId();
        // No installment Id found, generate a new one
        if (this.installmentId === '') {
            this.installmentId = await generateInstallmentId();
            return this.installmentId;
        }
        // Check if installment Id is still stamp
        const osStamp = calculateOsStamp();
        if (!this.installmentId.includes(osStamp)) {
            // If osStamp has change, regenerate new installment
            this.installmentId = await generateInstallmentId();
            return this.installmentId;
        }
        return this.installmentId;
    }
    async trackEvent(command = '', properties = {}) {
        // if (!this.session.opts.collectMetrics) return
        if (!command || command.length === 0)
            return;
        const installmentId = await this.getInstallmentId();
        const user = this.getUserProperties();
        const body = {
            installmentId,
            platform: this.osTraits,
            user,
            command,
            properties,
        };
        return api_client_1.req.post(uris_1.ANALYTICS_API_PATH, body, { timeout: 2000 });
    }
}
exports.default = Analytics;
